# Assembly (Сборки)
## Зачем?
Как и в случае с модулями, в какойто момент и модулей может стать очень много, тогда на помощью приходят сборки. В отличии от модулей, сборка является более сложным типом, и содержит зависимости на другие сборки.

## Создание
Для создания сборки, нам надо отнаследоватся от протокола DIAssembly, и определить 3 переменных:
```Swift
class YourAssembly: DIAssembly {
  var publicModules: [DIModule] { return [...] }
  var modules: [DIModule] { return [...] }
  
  var dependencies: [DIAssembly] { return [...] }
}
```

Как мы видим у сборки есть 3 поля:
* publicModules - список всех публичных модулей. Отличие публичных модулей от обычных будет расмотрено ниже.
* modules - список всех обычных модулей.
* dependencies - список зависимостей на другие сборки.

## Публичный/обычный модуль

Чтобы понимать различие между этими понятиями, достаточно запомнить два правила:
* Публичный модуль включается в саму сборку, и во все сборки которые зависят от этой сборки.
* Обычный модуль включается в саму сборку, но не включается в сборки которые зависят от этой сборки.

## Как использовать
Сборки используются похожим образом, как и модуля. То есть мы должны просто добавить сборку в наш билдер. Делается это вот так:
```Swift
builder.register(assembly: YourAssembly())
```
## Динамические сборки
Иногда, так бывает, что сборка содержит внутри себя данные, которые будут известны только после того как будут созданы другие сборки. К примеру - вы решили вынести набор интерфейсов в отдельную сборку, но реализация этих интерфейсов разнесена по другим сборкам, которые общаются между друг другом по средствам этих интерфейсов. Это удобно, если вы не хотите жестко завязывать сборки друг на друга, а завязать через посредника, тем самым обеспечив возможность легко убирать часть сборок из вашего проекта (но надо понимать, что в этом случае использовать `try!` при разрешении зависимостей, опасно, так как часть зависимостей является опциональными, в этом случае можно использовать `try?`). Для таких случаев придуманы динамические сборки, которые являются обычными сборками с одним лишь дополнением - в них могут добавлятся модули из других сборок. 
Для того чтобы создать динамическую сборку, надо отнаследоваться от класса: `DIDynamicAssembly`, после чего вы можете делать с этой сборкой все тоже самое, что и с обычной. Для того чтобы добавить теперь модуль из другой сборки нужно воспользоватся методом: `add(module: module)`, все это вместе может выглядет тяк:
```Swift
class YourDynamicAssembly: DIDynamicAssembly {
  var publicModules: [DIModule] { return [...] }
  var modules: [DIModule] { return [...] }
  
  var dependencies: [DIAssembly] { return [...] }
}
...
class YourAssembly: DIAssembly {
  func addDynamicModules() {
    YourDynamicAssembly().add(module: YourModuleFromCurrentAssembly())
  }
}
```

## Нюансы
* Типы `Singleton` и `LazySingleton` единственные для всей программы, а не сборки.
* Наилучший подход 1 проект(подпроект) = 1 сборке, при условии что у вас внутри проекта есть подпроекты.
