# Быстрый старт

## Структура приложений
Идея заключается в том, чтобы инвертировать зависимости - не класс должен доставать зависимости из разных мест, а зависимости должны приходит в класс из вне. Тем самым классы могут заниматься тем чем и должны - выполнять свои функции, не заботясь о том откуда ему взять интересующие его данные.

За частую проекты состоят из следующих модулей:
View - отвечают за отображение данных на экран, и считывание данных с периферийных устройств.
Controller - отвечают за взаимодействия Model и View. В разных концепциях Controller может называться по разному, но суть его везде остается одна и та же.
Model - в неё входят бизнес правил, и правила взаимодействия с данными. За частую этот модуль делится на более мелкие модуля.
Router - взаимодействует с Controller-ами и отвечает за ихнюю смену.

Если рассматривать iOS приложение, то происходит некоторое слияние понятий View и Controller, что не очень хорошо и модель становится больше похожа на:
ViewController - это xib файлы (или Storyboard окна) + классы наследуемые от UIViewController. Такое слияние приводит к тому, что анимация или еще некоторые данные которые относятся к отображению, перекочёвывают в Controller.
Model - набор классов с которыми взаимодействуют ViewController.
Router - это Storyboard, или некоторый собственный класс который обеспечивает смену окон.

Но во всех этих моделях, есть одна главная часть - все модули взаимодействую между друг другом. Для взаимодействия была придумана методология - IoC контейнеры.

Суть их заключается в том, что где-то, обычно при запуске приложения, прописываются все взаимодействия между модулями/классами, а контейнер берет на себя обязанность за создание/удаление этих классов, и разрешение зависимостей.
 
## Добавление DITranquallity в ваш проект
DITranquallity являет проектом с открытым исходным кодом, и поддерживает cocoapods. 
Самый простой способ подключить его в проект, это воспользоваться cocoapods:  
`pod 'DITranquallity'` - подключает базовые классы необходимые для работы с библиотекой  
`pod 'DITranquallity/Storyboard'` - нужен для автоматизации разрешения зависимостей у ViewController-ах на Storyboard.

## Регистрация зависимостей
Для регистрации зависимостей вы должны вначале создать билдер, после чего прописать все зависимости и их свойства в него, и потом сконструировать контейнер.
В DITranquallity это делается следующим образом:
```Swift
let builder = DIContainerBuilder()

builder.register(Cat.self).asSelf().initializer { Cat(name: "Felix") }
builder.register(Dog.self).asSelf().initializer { Dog(name: "Buddy") }
builder.register(Home.self).asSelf().initializer { (s) in return Home(animals: [s.resolve(Cat), s.resolve(Dog)]) }

let container = try! builder.build()
```

## Разрешение зависимостей
Для разрешения зависимостей вы пользуетесь контейнером, и спрашиваете у него какие зависимости вас интересуют.
В DITranquallity это делается следующим образом:
```Swift
let cat = try! container.resolve(Cat.self)
let dog = try! container.resolve(Dog.self)
let home: Home = try! container.resolve()

print(cat.name) //Felix
print(dog.name) //Buddy
print(home.animals)// [Cat, Dog]
print(home.animals.map{ $0.name })// [Felix, Buddy]
```

## Что дальше?
Более подробную информацию можно прочитать в следующих главах:

* [Базовая регистрации зависимостей](registration.md)
* [Создание контейнера](build.md)
* [Время жизни](timelife.md)
* [Регистрация с указанием имени/множественная регистрация](multi_name_registration.md)
* [Разрешение зависимостей](resolve.md)
* [Модули](module.md)
* [Сборки](assembly.md)
* [Storyboard](storyboard.md)
* [Примеры](sample.md)